package Src;

public class Vtas {
	private InfoTas[] vTas;
	private int[] pos;
	private int dernierIndice = 0;
	
	public Vtas(int nbSommet) {
		vTas = new InfoTas[nbSommet+1];
		pos = new int[nbSommet];
	} 
	
	//insertion de sommets dans le tas
	public void inserer(int numSom, float cout){
		//On le met a la fin temporairement
		vTas[dernierIndice]= new InfoTas(numSom,cout);
		this.pos[numSom] = dernierIndice;
		//On verifie que le sommet est bien a sa place
		if(dernierIndice > 0)
			this.moveUp(dernierIndice);
        //On augmente l'indice de sommet de 1
		dernierIndice++;
	}
	
	//retourne l'element en haut du tas
	public InfoTas getTete(){
		return vTas[0];
	}
	
	//vire la tete et fait remonter la nouvelle tete
	public void retirerTete(){
		echange(0,dernierIndice-1); //on remplace la tete du tas par son dernier ?l?ment
		dernierIndice--; // on supprime en quelque sorte l'ancienne tete
		moveDown(0); // et on reconstruit le tas
	}
	
	
		public void print(){ //methode pour les tests uniquement
		for(int i=0;i<dernierIndice;i++){
			System.out.print(+vTas[i].getIdSom()+"["+vTas[i].getCout()+"/"+vTas[i].getNivBat()+"]-");
		}
		System.out.println();
	}
	
		//retourne le nb d'elements du tas
	public int taille(){
		return dernierIndice-1;
	}
	
	//methode pour retourner les infos d'un sommet en fonction de son numero
	public InfoTas getSommet(int numSommet){
		return vTas[pos[numSommet]];
	}
	
	//surcharge de la methode precedente pour l'insertion de sommets prenant en compte le niveau de batterie
	public void inserer(int numSom,float cout, double nivBat){
		vTas[dernierIndice]= new InfoTas(numSom,cout,nivBat);
		this.pos[numSom] = dernierIndice;
		if(dernierIndice > 0)
			this.moveUp(dernierIndice);
		dernierIndice++;
	}
	
	public boolean appartient(int numSommet){
		if(vTas[0].getIdSom() == numSommet) //si le sommet qu'on demande est a la tete du tas (indice a 0) 
			return true;
		return (pos[numSommet] > 0); //sinon on retourne vrai si la position est > 0  faux sinon
	}
	
	public boolean marque(int numSommet){ //permet de savoir si un sommet est marque (deja ete retire du tas)
		return (pos[numSommet] >= dernierIndice);
	}
	
	public boolean estVide(){
		return (dernierIndice == 0);
	}
	
	
	/**
	 * Met a jour le cout d'un sommet (en mettant a jour l'arborescence du tas)
	 * @param som : Sommet que l'on veux mettre a jour
	 * @param nouvelleVal : Nouveau cout pour le sommet
	 */
	public void maj(int numSommet, float cout){
		//Mise a jour du cout
		vTas[pos[numSommet]].setCout(cout);
		//Si le fils a un cout inferieur au pere alors on le deplace vers le haut dans l'arborescence
		if (vTas[pos[numSommet]].getCout() < vTas[(pos[numSommet]/2)].getCout())
			moveUp(pos[numSommet]);
		else   //Sinon on le deplace vers le bas
			moveDown(pos[numSommet]);
		
	}
	
	//surcharge pour la mise a jour d'un sommet et du niveau de batterie correspondant
	public void maj(int numSommet, float cout, double nivBat){
		//Mise a jour du cout
		vTas[pos[numSommet]].setCout(cout);
		vTas[pos[numSommet]].setNivBat(nivBat);
		//Si le fils a un cout inferieur au pere alors on le deplace vers le haut dans l'arborescence
		if (vTas[pos[numSommet]].getCout() < vTas[(pos[numSommet]/2)].getCout())
			moveUp(pos[numSommet]);
		else   //Sinon on le deplace vers le bas
			moveDown(pos[numSommet]);
		
	}
	
	//fait ?voluer un sommet vers le haut du tas si besoin
	private void moveUp(int indice){
		if(indice > 0){
			int demiIndice = indice/2;
			if(vTas[demiIndice].getCout() > vTas[indice].getCout()) {//si c'est le cas il faut faire evoluer, sinon le sommet est a sa place
				echange(indice,demiIndice);
				//on rappelle la fonction pour continuer l'histoire
				moveUp(demiIndice);
			}
		}		
	}
	
	//fait ?voluer un sommet vers le bas du tas si besoin
	private void moveDown(int indice){
		int doubleIndice = indice * 2;
		if(doubleIndice+1 < dernierIndice){ //si on peut faire descendre plus l'?lement
			
			//s'il y a besoin de faire descendre l'element
			if((vTas[indice].getCout() > vTas[doubleIndice].getCout()) || (vTas[indice].getCout() > vTas[doubleIndice+1].getCout())) {
				
				if(vTas[doubleIndice].getCout() <= vTas[doubleIndice+1].getCout()) {//si le fils gauche est plus petit que le droit
					echange(indice,doubleIndice);
					moveDown(doubleIndice);
				}
				
				else { //sinon on fait l'echange avec le fils droit
					echange(indice,doubleIndice+1);
					moveDown(doubleIndice+1);
				}
				
			}
					
		}
		else if(doubleIndice == (dernierIndice -1)){//cas particulier ou l'element n'a qu'un fils gauche
			if(vTas[indice].getCout() > vTas[doubleIndice].getCout())
				echange(indice,doubleIndice);
		}
	}
	
	
	
	private void echange(int indice1, int indice2){
		int auxNum = vTas[indice1].getIdSom();
		float auxVal = vTas[indice1].getCout();
		double auxNivBat = vTas[indice1].getNivBat();
		
		vTas[indice1].setIdSom(vTas[indice2].getIdSom());
		vTas[indice1].setCout(vTas[indice2].getCout());
		vTas[indice1].setNivBat(vTas[indice2].getNivBat());
		vTas[indice2].setIdSom(auxNum); //inversion des deux elements
		vTas[indice2].setCout(auxVal); 	
		vTas[indice2].setNivBat(auxNivBat);
		//mise a jour des positions
		pos[vTas[indice1].getIdSom()] = indice1;
		pos[vTas[indice2].getIdSom()] = indice2;
	}

}
